<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <title>Passaporte Web :: {% block title %}{% endblock title %}</title>
    <meta name="description" content="">
    <meta name="author" content="">

    <!-- Le HTML5 shim, for IE6-8 support of HTML elements -->
    <!--[if lt IE 9]>
      <script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
    <![endif]-->
    <script src="{{ MEDIA_URL }}bootstrap/js/bootstrap-alerts.js"></script>
    {% block scripts %}{% endblock scripts %}

    <!-- Le styles -->
    <link href="{{ MEDIA_URL }}bootstrap/bootstrap.css" rel="stylesheet">
    <style type="text/css">
      /* Override some defaults */
      html, body {
        background-color: #eee;
      }
      body {
        padding-top: 32px; /* 40px to make the container go all the way to the bottom of the topbar */
      }
      .container > footer p {
        text-align: center; /* center align it with the container */
      }
      .container {
        width: 820px; /* downsize our container to make the content feel a bit tighter and more cohesive. NOTE: this removes two full columns from the grid, meaning you only go to 14 columns and not 16. */
      }

      /* The white background content wrapper */
      .content {
        background-color: #fff;
        padding: 20px;
        margin: 0 -20px; /* negative indent the amount of the padding to maintain the grid system */
        -webkit-border-radius: 0 0 6px 6px;
           -moz-border-radius: 0 0 6px 6px;
                border-radius: 0 0 6px 6px;
        -webkit-box-shadow: 0 1px 2px rgba(0,0,0,.15);
           -moz-box-shadow: 0 1px 2px rgba(0,0,0,.15);
                box-shadow: 0 1px 2px rgba(0,0,0,.15);
      }

      /* Page header tweaks */
      .page-header {
        background-color: #f5f5f5;
        padding: 20px 20px 10px;
        margin: -20px -20px 20px;
      }

      /* Styles you shouldn't keep as they are for displaying this base example only */
      .content .span10,
      .content .span4 {
        min-height: 500px;
      }
      /* Give a quick and non-cross-browser friendly divider */
      .content .span4 {
        margin-left: 0;
        padding-left: 19px;
        border-left: 1px solid #eee;
      }

    </style>
  </head>

  <body>

    <div class="container">

      <div class="content">
        {% block content %}
        <div class="page-header">
          {% block content-header %}
          <h1>Page name <small>Supporting text or tagline</small></h1>
          {% endblock content-header %}
        </div>
        <div class="row">
          <div class="span10">
            {% block content-inner %}
            <h2>Main content</h2>
            {% endblock content-inner %}
          </div>
          <div class="span4">
            {% block content-aside %}
            <h3>Secondary content</h3>
            {% endblock content-aside %}
          </div>
        </div>
        {% endblock content %}
      </div>

      <footer>
        <p>Copyright&copy; 2011 Free Communities. Todos os direitos reservados.</p>
      </footer>

    </div>

  </body>
    <script type="text/javascript" src="{{ PASSAPORTE_WEB_HOST }}/navbar/identity_client.js?login_url=/sso/&logout_url={{ APPLICATION_HOST }}/auth/logout/&push_it=false"></script>
</html>
