{% extends "base.html" %}
{% block title %}Login via API{% endblock %}

{% block content-header %}
  <h1>Login via API</h1>
{% endblock content-header %}

{% block content-inner %}
    <div class="login">
        {% if form.errors %}
        <div class="alert-message error">
          <a class="close" href="#">×</a>
          <p>
            <strong>Erro!</strong>
            Não foi possível efetuar o login, confira as informações e favor tente novamente.
          </p>
        </div>
        {% endif %}

        <form method="post" action=".">
          <fieldset>
            <div class="clearfix">
              <label for="xlInput">{{ form.email.label_tag }}</label>
              <div class="input">
                  {{ form.email }}
                  <span class="help-inline">{{ form.email.errors }}</span>
              </div>
            </div>
            <div class="clearfix">
              <label for="xlInput">{{ form.password.label_tag }}</label>
              <div class="input">
                  {{ form.password }}
                  <span class="help-inline">{{ form.password.errors }}</span>
              </div>
            </div>

            {% csrf_token %}
            <input type="hidden" name="next" value="{{ next }}" />
          </fieldset>

        </form>
    </div>
{% endblock content-inner %}

{% block content-aside %}
    <dl>
        <dt>Sobre esta operação <span class="label important">Importante</span></dt>
        <dd>Deve-se utilizar este método de login somente em casos especiais.</dd>
    </dl>
{% endblock content-aside %}
